#include <MegaServo.h>;
// Библиотека для работы с сервами
// https://playground.arduino.cc/Code/MegaServo

// пины PWM
#define THROTTLE_IN  5
#define LIGHTS_IN 6
#define RADAR_IN 3
#define THROTTLE_OUT 10
#define REVERSE_OUT 9

// константы для PWM
#define THROTTLE_MIN_IN 1000
#define THROTTLE_MAX_IN 2000
#define THROTTLE_MIN_OUT 800
#define THROTTLE_MAX_OUT 2300
#define SERVO_MAX_ANGLE 180
#define REVERSE_ON 90
#define REVERSE_OFF 30


#define DEAD_ZONE 15 // зона нечувствительности к курку в каждую сторону предполагаемого хода сервы. в градусах

// пины декораций
#define RADAR 4
#define LED1 20
#define LED_WHITE 19
#define LED_YELLOW 18
#define LED_BLUE 15
#define LED2 14
#define LED3 7
#define LED4 21

// время между включением и выключением синего огня
#define MAX_MILLS 500

//#define DEBUG

MegaServo throttle_servo, reverse_servo;

int thro_pwm[3] = {0, 0, 0}, 
    ptr=0, 
    old=0,
    mode1=0, m1_old=0, m1_buf=0,
    mode2=0, m2_old=0, m2_buf=0;

uint32_t mil=0;

int read_thro_pwm() {
  if (++ptr > 2) ptr = 0;
  int a = pulseIn(THROTTLE_IN, HIGH);

  thro_pwm[ptr] =  map(a, THROTTLE_MIN_IN, THROTTLE_MAX_IN, -SERVO_MAX_ANGLE, SERVO_MAX_ANGLE); 

  a = thro_pwm[0];
  int  b = thro_pwm[1], c = thro_pwm[2], middle;

  if ((a <= b) && (a <= c)) {
    middle = (b <= c) ? b : c;
  }
  else {
    if ((b <= a) && (b <= c)) {
      middle = (a <= c) ? a : c;
    }
    else {
      middle = (a <= b) ? a : b;
    }
  }
  if(middle>-DEAD_ZONE && middle<DEAD_ZONE) middle=0;
  return middle;
}

void setup() {
  pinMode(THROTTLE_IN, INPUT); 
  pinMode(LIGHTS_IN, INPUT);
  pinMode(RADAR, OUTPUT);
  pinMode(LED1, OUTPUT);
  pinMode(LED_WHITE, OUTPUT);
  pinMode(LED_YELLOW, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(LED3, OUTPUT);
  pinMode(LED4, OUTPUT);
  
  throttle_servo.attach(THROTTLE_OUT);
  reverse_servo.attach(REVERSE_OUT);
  reverse_servo.write(REVERSE_ON);
  digitalWrite(RADAR, HIGH);
  digitalWrite(LED1, HIGH);
  throttle_servo.writeMicroseconds(THROTTLE_MAX_OUT);
  delay(1000);
  digitalWrite(LED1, LOW);
  digitalWrite(LED_WHITE, HIGH);
  delay(1000);
  digitalWrite(LED_WHITE, LOW);
  digitalWrite(LED_YELLOW, HIGH);
  throttle_servo.writeMicroseconds(THROTTLE_MIN_OUT);
  delay(1000);
  digitalWrite(LED_YELLOW, LOW);
  digitalWrite(LED_BLUE, HIGH);
  delay(1000);
  digitalWrite(LED_BLUE, LOW);
  digitalWrite(LED2, HIGH);
  delay(1000);
  digitalWrite(LED2, LOW);
  digitalWrite(LED3, HIGH);
  delay(1000);
  digitalWrite(LED3, LOW);
  digitalWrite(LED4, HIGH);
  delay(1000);
  digitalWrite(LED4, LOW);
  digitalWrite(LED_BLUE, HIGH);
  delay(1000);
  digitalWrite(LED_BLUE, LOW);
  digitalWrite(RADAR, LOW);
  reverse_servo.write(REVERSE_OFF);
  #ifdef DEBUG
  Serial.begin(9600);
  Serial.print("Hello world!! ");
  #endif
}

void loop() {
  int x = read_thro_pwm(); 
  if(x!=old) {
    
        #ifdef DEBUG
        Serial.print("T ");
        Serial.print(x); 
        Serial.print(" "); 
        Serial.println(old);
        #endif
        
        if(x<0) {
          // назад
          reverse_servo.write(REVERSE_ON);
          x = abs(x)/2;
        } else {
          // вперед
          reverse_servo.write(REVERSE_OFF);
        }
        old = x;
        if(x==0) {
          throttle_servo.writeMicroseconds(THROTTLE_MIN_OUT);
        } else {
          x = map(x, 0, SERVO_MAX_ANGLE, THROTTLE_MIN_OUT, THROTTLE_MAX_OUT);
          throttle_servo.writeMicroseconds(x);
        }
  }
  x = pulseIn(LIGHTS_IN, HIGH);
  if (x>1700 && m1_old<1500) {
    m1_old = x;
    if (++mode1 > 2) mode1 = 0;
  }
  if (m1_old>1700 && x<1500) m1_old = x;
  
  #ifdef DEBUG
  Serial.print("L ");
  Serial.print(mode1);
  #endif
  
  x = pulseIn(RADAR_IN, HIGH);
  if (x>1700 && m2_old<1500) {
    m2_old = x;
    if (++mode2 > 4) mode2 = 0;
  }
  if (m2_old>1700 && x<1500) m2_old = x;
  
  #ifdef DEBUG
  Serial.print(" R ");
  Serial.println(mode2);
  #endif

  if(mode1 != m1_buf) {
    m1_buf = mode1;
    digitalWrite(LED1, LOW);
    digitalWrite(LED_WHITE, LOW);
    digitalWrite(LED_YELLOW, LOW);
    digitalWrite(LED_BLUE, LOW);
    digitalWrite(LED3, LOW);

    switch(mode1) {
      case 1:
        digitalWrite(LED1, HIGH);
        digitalWrite(LED_WHITE, HIGH);
        digitalWrite(LED_BLUE, HIGH);
        break;
      case 2:
        digitalWrite(LED3, HIGH);
        digitalWrite(LED_WHITE, HIGH);
        digitalWrite(LED_YELLOW, HIGH);
        break;
    }
  }

  if (millis() - mil >= MAX_MILLS) {   
    mil += MAX_MILLS;   // сброс таймера
    if(mode1==1){
      digitalWrite(LED_BLUE, !digitalRead(LED_BLUE));
    }
  }

  if(mode2 != m2_buf) {
    m2_buf = mode2;
    digitalWrite(RADAR, LOW);
    digitalWrite(LED2, LOW);
    digitalWrite(LED4, LOW);
    switch(mode2) {
      case 1:
        digitalWrite(RADAR, HIGH);
        break;
      case 2:
        digitalWrite(RADAR, HIGH);
        digitalWrite(LED2, HIGH);
        break;
      case 3:
        digitalWrite(RADAR, HIGH);
        digitalWrite(LED4, HIGH);
        break;  
      case 4:
        digitalWrite(LED2, HIGH);  
        break;
    }
  }
}
